unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, VCLTee.TeEngine,
  VCLTee.Series, Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart, VCLTee.TeeEdit;

type
  TForm1 = class(TForm)
    Image1: TImage;
    chrActTaxation: TChart;
    srsIncomeNeedByBracket: TPieSeries;
  private
    { Private declarations }
    procedure UpdateChartImage;
  protected
    procedure Resize; override;
  public
    constructor Create(aOwner: TComponent); override;

  end;

var
  Form1: TForm1;

implementation

uses
  System.Math, VCLTee.TeCanvas;

{$R *.dfm}

procedure ConfigChart(const aChart: TChart; const aSwapAxis: boolean);
var
 lAxis: TChartAxis;
begin
  aChart.Title.Font.Size := 12;
  aChart.Title.Font.Style := [fsBold];
  aChart.Shadow.Visible := false;
  aChart.BevelInner := bvNone;
  aChart.BevelOuter := bvNone;

  if aSwapAxis then
    lAxis := aChart.BottomAxis
  else
    lAxis := aChart.LeftAxis;

  lAxis.Title.Font.Size := 10;
  lAxis.Title.Font.Style := [fsBold];

  lAxis.LabelsFont.Size := 9;
  lAxis.LabelsSeparation := ifthen(aSwapAxis, 50, 100);
  lAxis.Axis.Width := 1;
  lAxis.Ticks.Visible := False;
  lAxis.MinorTicks.Visible := False;
  lAxis.TicksInner.Visible := False;
  lAxis.Grid.Color := clSilver;
  lAxis.Grid.Style := psDot;

  if aSwapAxis then
    lAxis := aChart.LeftAxis
  else
    lAxis := aChart.BottomAxis;

  lAxis.Title.Font.Size := 10;
  lAxis.Title.Font.Style := [fsBold];
  lAxis.Axis.Width := 1;
  lAxis.Grid.Visible := False;
  lAxis.MinorTicks.Visible := False;
  lAxis.TicksInner.Visible := False;

  lAxis.LabelsFont.Size := 9;
  lAxis.LabelsSeparation := ifthen(aSwapAxis, 100, 50);

  aChart.Color := clWhite;
  aChart.RightAxis.Visible := False;
  aChart.TopAxis.Visible := False;
  aChart.BackWall.Visible := False;

  aChart.LeftWall.Color := clWhite;

  aChart.Legend.Font.Size := 9;
  aChart.Legend.Symbol.Squared := True;
  aChart.Legend.Symbol.Pen.Visible := False;
  aChart.Legend.Symbol.Shadow.Visible := False;
  aChart.Legend.Gradient.Visible := False;
  aChart.Legend.Shadow.Visible := False;
  aChart.Legend.Frame.Visible := False;
  aChart.Legend.Transparent := True;

  aChart.View3D := False;
  aChart.Gradient.Visible := False;
  aChart.Zoom.Allow := false;
end;


procedure ConfigMarks(const aMarks: TSeriesMarks);
begin
//  aMarks.BackColor := MarksBackColor;
//  aMarks.Font.Color := MarkFontColor;
  aMarks.Font.Size := 9;
  aMarks.Transparent := True;
end;

procedure AssignChartToGraphic(aChart: TChart; const aRect: TRect; aGraphic: TGraphic; aTransparent: boolean = false);
var
  lHoldShadow: boolean;
  I: Integer;
  lBoolArray: array of boolean;
  lHoldBevel: TBevelCut;
  lHoldColor: TColor;
  lHoldGradientVisible: boolean;
  lMetafile: TMetafile;
  {$ifopt C+}
  lCanvasClassName: string;
  {$endif}
  lBmp: TBitmap;
  lCanvasClass: TCanvas3DClass;
  lHoldHover: Boolean;
begin
  {$ifopt C+}
  lCanvasClassName := aChart.Canvas.ClassName;
  {$endif}
  lCanvasClass := TCanvas3DClass(aChart.Canvas.ClassType);
  if lCanvasClass <> TTeeCanvas3D then
    aChart.Canvas := TTeeCanvas3D.Create;

  lHoldGradientVisible := aChart.Gradient.Visible;
  lHoldColor := aChart.Color;
  lHoldBevel := aChart.BevelOuter;
  lHoldShadow := aChart.Shadow.Visible;
  lHoldHover := aChart.Hover.Visible;

  try
    SetLength(lBoolArray, aChart.SeriesCount);
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
      begin
        lBoolArray[I] := TPieSeries(aChart.Series[I]).Circled;
        TPieSeries(aChart.Series[I]).Circled := True;
      end
      else
        lBoolArray[I] := True;


    aChart.Gradient.Visible  := false;
    if not aTransparent then
      aChart.Color := clWhite
    else
      aChart.Color := VCL.Graphics.clNone; //clWhite;
    aChart.BevelOuter := bvNone;
    aChart.BevelInner := bvNone;
    aChart.Shadow.Visible := False;
    aChart.Hover.Visible := false;

    if aGraphic is TMetafile then
    begin
      lMetafile := aChart.TeeCreateMetafile(false, aRect);
      try
        aGraphic.Assign(lMetafile);
      finally
        lMetafile.Free;
      end;
    end
    else
    begin
      lBmp := aChart.TeeCreateBitmap(clWhite, aRect);
      try
        aGraphic.Assign(lBmp);
      finally
        lBmp.Free;
      end;
    end;
  finally
    aChart.Hover.Visible := lHoldHover;
    aChart.Shadow.Visible := lHoldShadow;
    aChart.Gradient.Visible := lHoldGradientVisible;
    aChart.Color := lHoldColor;
    aChart.BevelOuter := lHoldBevel;
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
        TPieSeries(aChart.Series[I]).Circled := lBoolArray[I];
    if (lCanvasClass <> aChart.Canvas.ClassType) then
      aChart.Canvas := lCanvasClass.Create;
  end;

  {$ifopt C+}
  assert(SameText(lCanvasClassName, aChart.Canvas.ClassName),
    Format('Chart canvas should be %s but is %s', [lCanvasClassName, aChart.Canvas.ClassName]));
  {$endif}
end;



{ TForm1 }

constructor TForm1.Create(aOwner: TComponent);
begin
  inherited;

  ConfigChart(chrActTaxation, false);
  ConfigMarks(srsIncomeNeedByBracket.Marks);

  srsIncomeNeedByBracket.Clear;
  srsIncomeNeedByBracket.AddPie(407400, '10%'#13#10'$999,999');
  srsIncomeNeedByBracket.AddPie(425586, '12%'#13#10'$999,999');
  srsIncomeNeedByBracket.AddPie(20763, '22%'#13#10'$999,999');
  srsIncomeNeedByBracket.AddPie(2375, '22%'#13#10'$999,999');


end;

procedure TForm1.Resize;
begin
  inherited;
  Image1.Width := ClientWidth div 2;
  Caption := Format('Width: %d; Height: %d', [Width, Height]);

  UpdateChartImage;
end;

procedure TForm1.UpdateChartImage;
var
  lBmp: TBitmap;
  lRect: TRect;
begin
  lBmp := TBitmap.Create;
  try
    lRect := Rect(0, 0, Image1.Width, Image1.Height);
    AssignChartToGraphic(chrActTaxation, lRect, lBmp);
    Image1.Picture.Graphic := lBmp;
  finally
    lBmp.Free;

  end;
end;

end.
